@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2012 Bastian Kleineidam
+# Copyright (C) 2000-2013 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,6 +32,8 @@ if _dnspath not in sys.path:
     sys.path.insert(0, _dnspath)
 del _dnspath
 import re
+import signal
+import traceback
 
 from . import i18n
 import _LinkChecker_configdata as configdata
@@ -176,3 +178,19 @@ def find_third_party_modules ():
         sys.path.append(os.path.join(third_party, "dnspython"))
 
 find_third_party_modules()
+
+if hasattr(signal, "SIGUSR1"):
+    # install SIGUSR1 handler
+    from .decorators import signal_handler
+    @signal_handler(signal.SIGUSR1)
+    def print_threadstacks(sig, frame):
+        """Print stack traces of all running threads."""
+        log.warn(LOG_THREAD, "*** STACKTRACE START ***")
+        for threadId, stack in sys._current_frames().items():
+            log.warn(LOG_THREAD, "# ThreadID: %s" % threadId)
+            for filename, lineno, name, line in traceback.extract_stack(stack):
+                log.warn(LOG_THREAD, 'File: "%s", line %d, in %s' % (filename, lineno, name))
+                line = line.strip()
+                if line:
+                    log.warn(LOG_THREAD, "  %s" % line)
+        log.warn(LOG_THREAD, "*** STACKTRACE END ***")
