@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2006-2010 Bastian Kleineidam
+# Copyright (C) 2006-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,45 +25,57 @@ from ..lock import get_lock
 _lock = get_lock("cookie")
 
 class CookieJar (object):
-    """
-    Cookie storage, implementing the default cookie handling policy for
-    LinkChecker.
-    """
+    """Cookie storage, implementing the cookie handling policy."""
 
     def __init__ (self):
-        self.cache = {}
+        """Initialize empty cookie cache."""
+        # Store all cookies in a set.
+        self.cache = set()
 
     @synchronized(_lock)
     def add (self, headers, scheme, host, path):
         """Parse cookie values, add to cache."""
-        jar = self.cache.setdefault(host, set())
+        errors = []
         for h in headers.getallmatchingheaders("Set-Cookie"):
             # RFC 2109 (Netscape) cookie type
             try:
-                jar.add(cookies.NetscapeCookie(h, scheme, host, path))
-            except cookies.CookieError:
-                log.debug(LOG_CACHE,
-               "Invalid cookie header for %s:%s%s: %r", scheme, host, path, h)
+                cookie = cookies.NetscapeCookie(h, scheme, host, path)
+                if cookie in self.cache:
+                    self.cache.remove(cookie)
+                if not cookie.is_expired():
+                    self.cache.add(cookie)
+            except cookies.CookieError as msg:
+                errmsg = "Invalid cookie %r for %s:%s%s: %s" % (
+                         h, scheme, host, path, msg)
+                errors.append(errmsg)
         for h in headers.getallmatchingheaders("Set-Cookie2"):
             # RFC 2965 cookie type
             try:
-                jar.add(cookies.Rfc2965Cookie(h, scheme, host, path))
-            except cookies.CookieError:
-                log.debug(LOG_CACHE,
-              "Invalid cookie2 header for %s:%s%s: %r", scheme, host, path, h)
-        self.cache[host] = jar
-        return jar
+                cookie = cookies.Rfc2965Cookie(h, scheme, host, path)
+                if cookie in self.cache:
+                    self.cache.remove(cookie)
+                if not cookie.is_expired():
+                    self.cache.add(cookie)
+            except cookies.CookieError as msg:
+                errmsg = "Invalid cookie2 %r for %s:%s%s: %s" % (
+                         h, scheme, host, path, msg)
+                errors.append(errmsg)
+        return errors
 
     @synchronized(_lock)
     def get (self, scheme, host, port, path):
-        """Cookie cache getter function."""
-        jar = self.cache.setdefault(host, set())
-        cookies = [x for x in jar if x.check_expired() and \
+        """Cookie cache getter function. Return ordered list of cookies
+        which match the given host, port and path.
+        Cookies with more specific paths are listed first."""
+        cookies = [x for x in self.cache if x.check_expired() and \
                    x.is_valid_for(scheme, host, port, path)]
+        # order cookies with more specific (ie. longer) paths first
+        cookies.sort(key=lambda c: len(c.attributes['path']), reverse=True)
         log.debug(LOG_CACHE, "Found %d cookies for host %r path %r",
                   len(cookies), host, path)
         return cookies
 
     @synchronized(_lock)
     def __str__ (self):
+        """Return stored cookies as string."""
         return "<CookieJar with %s>" % self.cache
