@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2012 Bastian Kleineidam
+# Copyright (C) 2000-2013 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -526,14 +526,17 @@ class UrlBase (object):
             self.check_connection()
             self.add_size_info()
             self.add_country_info()
-        except tuple(ExcList):
+        except tuple(ExcList) as exc:
             value = self.handle_exception()
             # make nicer error msg for unknown hosts
-            if isinstance(value, socket.error) and value.args[0] == -2:
+            if isinstance(exc, socket.error) and exc.args[0] == -2:
                 value = _('Hostname not found')
             # make nicer error msg for bad status line
-            if isinstance(value, httplib.BadStatusLine):
+            elif isinstance(exc, httplib.BadStatusLine):
                 value = _('Bad HTTP response %(line)r') % {"line": str(value)}
+            elif isinstance(exc, UnicodeError):
+                # idna.encode(host) failed
+                value = _('Bad hostname %(host)r: %(msg)s') % {'host': self.host, 'msg': str(value)}
             self.set_result(unicode_safe(value), valid=False)
         self.checktime = time.time() - check_start
         if self.do_check_content:
@@ -1094,6 +1097,8 @@ class UrlBase (object):
             app = winutil.get_word_app()
             try:
                 doc = winutil.open_wordfile(app, filename)
+                if doc is None:
+                    raise winutil.Error("could not open word file %r" % filename)
                 try:
                     for link in doc.Hyperlinks:
                         self.add_url(link.Address, name=link.TextToDisplay)
