@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2006-2012 Bastian Kleineidam
+# Copyright (C) 2006-2013 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -214,7 +214,7 @@ def abort_now ():
 def get_aggregate (config):
     """Get an aggregator instance with given configuration."""
     _urlqueue = urlqueue.UrlQueue(max_allowed_puts=config["maxnumurls"])
-    connections = connection.ConnectionPool(wait=config["wait"])
+    connections = connection.ConnectionPool(wait=config["wait"], limits=config["connectionlimits"])
     cookies = cookie.CookieJar()
     _robots_txt = robots_txt.RobotsTxt()
     return aggregator.Aggregate(config, _urlqueue, connections,
