@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2005-2011 Bastian Kleineidam
+# Copyright (C) 2005-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,11 @@ import stat
 import fnmatch
 import mimetypes
 import tempfile
+import importlib
+from distutils.spawn import find_executable
 
+from .decorators import memoized
+from . import log, LOG_CHECK
 
 def write_file (filename, content, backup=False, callback=None):
     """Overwrite a possibly existing file with new content. Do this
@@ -62,9 +66,10 @@ def has_module (name):
     @rtype: bool
     """
     try:
-        exec "import %s" % name
+        importlib.import_module(name)
         return True
-    except ImportError:
+    except (OSError, ImportError):
+        # some modules (for example HTMLtidy) raise OSError
         return False
 
 
@@ -136,7 +141,7 @@ class Buffer (object):
 def get_mtime (filename):
     """Return modification time of filename or zero on errors."""
     try:
-        return os.stat(filename)[stat.ST_MTIME]
+        return os.path.getmtime(filename)
     except os.error:
         return 0
 
@@ -144,7 +149,7 @@ def get_mtime (filename):
 def get_size (filename):
     """Return file size in Bytes, or -1 on error."""
     try:
-        return os.stat(filename)[stat.ST_SIZE]
+        return os.path.getsize(filename)
     except os.error:
         return -1
 
@@ -160,6 +165,7 @@ else:
     FSCODING = "utf-8"
 
 def pathencode (path):
+    """Encode a path string with the platform file system encoding."""
     if isinstance(path, unicode) and not os.path.supports_unicode_filenames:
         path = path.encode(FSCODING, "replace")
     return path
@@ -178,23 +184,55 @@ def has_changed (filename):
     return mtime > _mtime_cache[key]
 
 
-mimedb = mimetypes.MimeTypes(strict=False)
+mimedb = None
+
+def init_mimedb():
+    """Initialize the local MIME database."""
+    global mimedb
+    try:
+        mimedb = mimetypes.MimeTypes(strict=False)
+    except StandardError as msg:
+        log.error(LOG_CHECK, "could not initialize MIME database: %s" % msg)
+        return
+    # For Opera bookmark files (opera6.adr)
+    add_mimetype(mimedb, 'text/plain', '.adr')
+    # To recognize PHP files as HTML with content check.
+    add_mimetype(mimedb, 'application/x-httpd-php', '.php')
+    # To recognize WML files
+    add_mimetype(mimedb, 'text/vnd.wap.wml', '.wml')
+
+
+def add_mimetype(mimedb, mimetype, extension):
+    """Add or replace a mimetype to be used with the given extension."""
+    # If extension is already a common type, strict=True must be used.
+    strict = extension in mimedb.types_map[True]
+    mimedb.add_type(mimetype, extension, strict=strict)
+
 
 # if file extension lookup was unsuccessful, look at the content
 PARSE_CONTENTS = {
     "text/html": re.compile(r'^(?i)<(!DOCTYPE html|html|head|title)'),
     "text/plain+opera": re.compile(r'^Opera Hotlist'),
+    "text/plain+chromium": re.compile(r'^{\s*"checksum":'),
     "text/plain+linkchecker": re.compile(r'(?i)^# LinkChecker URL list'),
 }
 
 def guess_mimetype (filename, read=None):
     """Return MIME type of file, or 'application/octet-stream' if it could
     not be determined."""
-    mime, encoding = mimedb.guess_type(filename, strict=False)
+    mime, encoding = None, None
+    if mimedb:
+        mime, encoding = mimedb.guess_type(filename, strict=False)
+    basename = os.path.basename(filename)
+    # Special case for Safari Bookmark files
+    if not mime and basename == 'Bookmarks.plist':
+        return 'application/x-plist+safari'
+    # Special case for Google Chrome Bookmark files.
+    if not mime and basename == 'Bookmarks':
+        mime = 'text/plain'
     # Mime type text/plain can be differentiated further with content reading.
-    if (mime == "text/plain" or not mime) and read is not None:
+    if mime == "text/plain" and read is not None:
         # try to read some content and do a poor man's file(1)
-        # XXX replace with file(1) on Unix systems
         try:
             data = read()[:30]
             for mime, ro in PARSE_CONTENTS.items():
@@ -210,8 +248,40 @@ def guess_mimetype (filename, read=None):
     return mime.strip().lower()
 
 
-def get_temp_file (**kwargs):
+def get_temp_file (mode='r', **kwargs):
     """Return tuple (open file object, filename) pointing to a temporary
     file."""
     fd, filename = tempfile.mkstemp(**kwargs)
-    return os.fdopen(fd), filename
+    return os.fdopen(fd, mode), filename
+
+
+def is_tty (fp):
+    """Check if is a file object pointing to a TTY."""
+    return (hasattr(fp, "isatty") and fp.isatty())
+
+
+@memoized
+def is_readable(filename):
+    """Check if file is a regular file and is readable."""
+    return os.path.isfile(filename) and os.access(filename, os.R_OK)
+
+
+def is_accessable_by_others(filename):
+    """Check if file is group or world accessable."""
+    mode = os.stat(filename)[stat.ST_MODE]
+    return mode & (stat.S_IRWXG | stat.S_IRWXO)
+
+
+@memoized
+def is_writable(filename):
+    """Check if
+    - the file is a regular file and is writable, or
+    - the file does not exist and its parent directory exists and is
+      writable
+    """
+    if not os.path.exists(filename):
+        parentdir = os.path.dirname(filename)
+        return os.path.isdir(parentdir) and os.access(parentdir, os.W_OK)
+    return os.path.isfile(filename) and os.access(filename, os.W_OK)
+
+init_mimedb()
