@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2001-2012 Bastian Kleineidam
+# Copyright (C) 2001-2013 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -218,6 +218,10 @@ class LinkFinder (TagFinder):
                 base = self.base_ref
             # note: value can be None
             value = unquote(attrs.get(attr))
+            if tag == 'link' and attrs.get('rel') == 'dns-prefetch':
+                if ':' in value:
+                    value = value.split(':', 1)[1]
+                value = 'dns:' + value.rstrip('/')
             # parse tag for URLs
             self.parse_tag(tag, attr, value, name, base)
         log.debug(LOG_CHECK, "LinkFinder finished tag %s", tag)
