@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2011 Bastian Kleineidam
+# Copyright (C) 2000-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@ import os
 import datetime
 import time
 import codecs
-from ..decorators import notimplemented
+import abc
 from .. import log, LOG_CHECK, strformat, dummy, configuration, i18n
 
 _ = lambda x: x
@@ -42,6 +42,7 @@ Fields = dict(
     checktime=_("Check time"),
     url=_("URL"),
     level=_("Level"),
+    modified=_("Modified"),
 )
 del _
 
@@ -70,22 +71,31 @@ class LogStatistics (object):
 
     def reset (self):
         """Reset all log statistics to default values."""
-        # number of logged urls
+        # number of logged URLs
         self.number = 0
-        # number of encountered errors
+        # number of encountered URL errors
         self.errors = 0
-        # number of errors that were printed
+        # number of URL errors that were printed
         self.errors_printed = 0
-        # number of warnings
+        # number of URL warnings
         self.warnings = 0
-        # number of warnings that were printed
+        # number of URL warnings that were printed
         self.warnings_printed = 0
+        # number of internal errors
+        self.internal_errors = 0
+        # the set of checked domains
         self.domains = set()
+        # link types
         self.link_types = ContentTypes.copy()
+        # URL length statistics
         self.max_url_length = 0
         self.min_url_length = 0
         self.avg_url_length = 0.0
         self.avg_number = 0
+        # download stats
+        self.downloaded_bytes = None
+        # cache stats
+        self.robots_txt_stats = None
 
     def log_url (self, url_data, do_print):
         """Log URL statistics."""
@@ -120,6 +130,10 @@ class LogStatistics (object):
             # calculate running average
             self.avg_url_length += (l - self.avg_url_length) / self.avg_number
 
+    def log_internal_error (self):
+        """Increase internal error count."""
+        self.internal_errors += 1
+
 
 class Logger (object):
     """
@@ -148,6 +162,7 @@ class Logger (object):
     * log_url(url_data)
         Log a checked URL. Called by log_filter_url if do_print is True.
     """
+    __metaclass__ = abc.ABCMeta
 
     def __init__ (self, **args):
         """
@@ -174,6 +189,8 @@ class Logger (object):
         self.output_encoding = encoding
         # how to handle codec errors
         self.codec_errors = "replace"
+        # Flag to see if logger is active. Can be deactivated on errors.
+        self.is_active = True
 
     def get_charset_encoding (self):
         """Translate the output encoding to a charset encoding name."""
@@ -217,6 +234,7 @@ class Logger (object):
                 "Could not open file %r for writing: %s\n"
                 "Disabling log output of %s", self.filename, msg, self)
             self.fd = dummy.Dummy()
+            self.is_active = False
         self.filename = None
 
     def create_fd (self):
@@ -232,9 +250,17 @@ class Logger (object):
         Flush and close the file output denoted by self.fd.
         """
         if self.fd is not None:
-            self.flush()
+            try:
+                self.flush()
+            except IOError:
+                # ignore flush errors
+                pass
             if self.close_fd:
-                self.fd.close()
+                try:
+                    self.fd.close()
+                except IOError:
+                    # ignore close errors
+                    pass
             self.fd = None
 
     def check_date (self):
@@ -266,17 +292,25 @@ class Logger (object):
         return strformat.wrap(text, width, **kwargs).lstrip()
 
     def write (self, s, **args):
-        """
-        Write string to output descriptor.
+        """Write string to output descriptor. Strips control characters
+        from string before writing.
         """
         if self.filename is not None:
             self.start_fileoutput()
         if self.fd is None:
             # Happens when aborting threads times out
-            log.warn(LOG_CHECK,
-                "writing to unitialized or closed file")
+            log.warn(LOG_CHECK, "writing to unitialized or closed file")
         else:
-            self.fd.write(s, **args)
+            try:
+                self.fd.write(s, **args)
+            except IOError:
+                msg = sys.exc_info()[1]
+                log.warn(LOG_CHECK,
+                    "Could not write to output file: %s\n"
+                    "Disabling log output of %s", msg, self)
+                self.close_fileoutput()
+                self.fd = dummy.Dummy()
+                self.is_active = False
 
     def writeln (self, s=u"", **args):
         """
@@ -341,6 +375,8 @@ class Logger (object):
                      {'url': configuration.Url})
         self.comment(_("Write comments and bugs to %(url)s") %
                      {'url': configuration.SupportUrl})
+        self.comment(_("Support this project at %(url)s") %
+                     {'url': configuration.DonateUrl})
         self.check_date()
 
     def write_outro (self):
@@ -351,14 +387,14 @@ class Logger (object):
              {"time": strformat.strtime(self.stoptime),
               "duration": strformat.strduration_long(duration)})
 
-    @notimplemented
+    @abc.abstractmethod
     def log_url (self, url_data):
         """
         Log a new url with this logger.
         """
         pass
 
-    @notimplemented
+    @abc.abstractmethod
     def end_output (self):
         """
         End of output, used for cleanup (eg output buffer flushing).
@@ -389,7 +425,29 @@ class Logger (object):
             except (IOError, AttributeError):
                 pass
 
-# note: don't confuse URL loggers with application logs above
+    def log_internal_error (self):
+        """Indicate that an internal error occurred in the program."""
+        log.warn(LOG_CHECK, "internal error occurred")
+        self.stats.log_internal_error()
+
+    def add_statistics(self, robots_txt_stats, download_stats):
+        """Add cache and download statistics."""
+        self.stats.robots_txt_stats = robots_txt_stats
+        self.stats.downloaded_bytes = download_stats
+
+    def format_modified(self, modified, sep=" "):
+        """Format modification date if it's not None.
+        @param modified: modification date
+        @ptype modified: datetime or None
+        @return: formatted date or empty string
+        @rtype: unicode
+        """
+        if modified is not None:
+            return modified.isoformat(sep)
+        return u""
+
+
+# the standard URL logger implementations
 from .text import TextLogger
 from .html import HtmlLogger
 from .gml import GMLLogger
@@ -400,9 +458,10 @@ from .blacklist import BlacklistLogger
 from .gxml import GraphXMLLogger
 from .customxml import CustomXMLLogger
 from .none import NoneLogger
+from .sitemapxml import SitemapXmlLogger
 
 
-# default link logger classes
+# default URL logger classes
 Loggers = {
     "text": TextLogger,
     "html": HtmlLogger,
@@ -413,6 +472,7 @@ Loggers = {
     "blacklist": BlacklistLogger,
     "gxml": GraphXMLLogger,
     "xml": CustomXMLLogger,
+    "sitemap": SitemapXmlLogger,
     "none": NoneLogger,
 }
 # for easy printing: a comma separated logger list
