@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2009 Bastian Kleineidam
+# Copyright (C) 2000-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -64,15 +64,22 @@ class CustomXMLLogger (xmllog.XMLLogger):
             self.xml_tag(u"dlsize", u"%d" % url_data.dlsize)
         if url_data.checktime and self.has_part("checktime"):
             self.xml_tag(u"checktime", u"%f" % url_data.checktime)
+        if self.has_part("level"):
+            self.xml_tag(u"level", u"%d" % url_data.level)
         if url_data.info and self.has_part('info'):
             self.xml_starttag(u"infos")
             for info in url_data.info:
                 self.xml_tag(u"info", info)
             self.xml_endtag(u"infos")
+        if url_data.modified and self.has_part('modified'):
+            self.xml_tag(u"modified", self.format_modified(url_data.modified))
         if url_data.warnings and self.has_part('warning'):
             self.xml_starttag(u"warnings")
-            for data in url_data.warnings:
-                self.xml_tag(u"warning", data)
+            for tag, data in url_data.warnings:
+                attrs = {}
+                if tag:
+                    attrs["tag"] = tag
+                self.xml_tag(u"warning", data, attrs)
             self.xml_endtag(u"warnings")
         if self.has_part("result"):
             attrs = {}
