@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2010-2012 Bastian Kleineidam
+# Copyright (C) 2010-2013 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -84,7 +84,8 @@ def close_word_app (app):
 
 def open_wordfile (app, filename):
     """Open given Word file with application object."""
-    return app.Documents.Open(filename)
+    return app.Documents.Open(filename, ReadOnly=True,
+      AddToRecentFiles=False, Visible=False, NoEncodingDialog=True)
 
 
 def close_wordfile (doc):
